#pragma once

#include "Vector3.h"
#include "SnowAvalancheSim.h"

class Camera
{
public:
    Camera();
    virtual ~Camera();

    void click(Scene* pScene);

    inline void setEye(float x, float y, float z);
    inline void setEye(const Vector3& eye);
    inline void setUp(float x, float y, float z);
    inline void setUp(const Vector3& up);
    inline void setViewDir(float x, float y, float z);
    inline void setViewDir(const Vector3& vd);
    inline void setLookAt(float x, float y, float z);
    inline void setLookAt(const Vector3& look);
    inline void setBGColor(float x, float y, float z);
    inline void setBGColor(const Vector3& color);
    inline void setFOV(float fov) {m_fov = fov;}

    inline float fov() const                {return m_fov;}
    inline const Vector3 & viewDir() const  {return m_viewDir;}
    inline const Vector3 & lookAt() const   {return m_lookAt;}
    inline const Vector3 & up() const       {return m_up;}
    inline const Vector3 & eye() const      {return m_eye;}
    inline const Vector3 & bgColor() const  {return m_bgColor;}

    void drawGL();

private:

    void calcLookAt();

    Vector3 m_bgColor;

    // main screen params
    Vector3 m_eye;
    Vector3 m_up;
    Vector3 m_viewDir;
    Vector3 m_lookAt;
    float m_fov;

	DWORD m_dwFrames;
	DWORD m_dwCurrentTime;
	DWORD m_dwLastUpdateTime;
	DWORD m_dwElapsedTime;
};

extern Camera * g_camera;

//--------------------------------------------------------

inline void Camera::setEye(float x, float y, float z)
{
    m_eye.set(x, y, z);
}

inline void Camera::setEye(const Vector3& eye)
{
    setEye(eye.x, eye.y, eye.z);
}

inline void Camera::setUp(float x, float y, float z)
{
    m_up.set(x, y, z);
    m_up.normalize();
}

inline void Camera::setUp(const Vector3& up)
{
    setUp(up.x, up.y, up.z);
}

inline void Camera::setViewDir(float x, float y, float z)
{
    m_viewDir.set(x, y, z);
    m_viewDir.normalize();
}

inline void Camera::setViewDir(const Vector3& vd)
{
    setViewDir(vd.x, vd.y, vd.z);
}

inline void Camera::setLookAt(float x, float y, float z)
{
    Vector3 dir = Vector3(x, y, z) - m_eye;
    setViewDir(dir);
}

inline void Camera::setLookAt(const Vector3& vd)
{
    setLookAt(vd.x, vd.y, vd.z);
}

inline void Camera::setBGColor(float x, float y, float z)
{
    m_bgColor.set(x, y, z);
}

inline void Camera::setBGColor(const Vector3& vd)
{
    setBGColor(vd.x, vd.y, vd.z);
}

