#pragma once

#define __NO_STD_VECTOR // Use cl::vector instead of STL version
#define __CL_ENABLE_EXCEPTIONS

#include "SnowAvalancheSim.h"
#include "SystemTypes.h"
#include "Vector3.h"
#include "Vector4.h"
#include "clAdmin.h"
#include "clSort.h"
#include "Scene.h"
#include <cassert>

class clParticles
{
public:
	clParticles(uint numParticles, float gridSize, float particleRadius);
	~clParticles(void);

	enum ParticleConfig {
		CONFIG_RANDOM,
		CONFIG_GRID
	};

	uint getNumParticles() { return m_numParticles; }
	uint getPosVBO() { return m_posVBO; }
	uint getColorVBO() { return m_colorVBO; }
	float getParticleRadius() { return m_params.particleRadius; }

	void init();
	void close();

	void reset(ParticleConfig config);

	void createProgram();
	void update();

protected: // methods
	void sort();
	void copyDataToGPU(const void * cpuData, cl::Buffer gpuData, ::size_t size);
	void copyDataFromGPU(void * cpuData, cl::Buffer gpuData, ::size_t size);

	void initializeDensity(float m_snow, float m_mixture);
	void calcHash();
	void findCellBounds();
	void computeDensity();
	void computePressure();
	void computeForces();
	void integrate();

private:
	uint m_numParticles;
	bool m_initialized;
	cl::Program m_clParticlesProgram;
	clSort* m_sorter;

	cl::Kernel kInitializeDensity;
	cl::Kernel kCalcHash;
	cl::Kernel kFindCellBounds;
	cl::Kernel kComputeDensity;
	cl::Kernel kComputePressure;
	cl::Kernel kComputeForces;
	cl::Kernel kIntegrate;

	// CPU data
	float* cVel;
	float* cPos;
	float* cSortedVel;
	float* cSortedPos;

	float* cSimVar;
	float* cParVar;

	uint* cCellStart;
	uint* cCellEnd;
	uint* cIndex;
	uint* cHash;
	simParams_t m_params;

	///
	float* cForce;
	float* cInitDen;

	// GPU data
	cl::Buffer bVel;
	cl::BufferGL bPos;
	cl::Buffer bSortedVel;
	cl::Buffer bSortedPos;
	cl::BufferGL bColor;

	cl::Buffer bSimVar;
	cl::Buffer bParVar;

	cl::Buffer bCellStart;
	cl::Buffer bCellEnd;
	cl::Buffer bIndex;
	cl::Buffer bHash;
	cl::Buffer bParams;

	///

	cl::Buffer bForce;
	cl::Buffer bInitDen;

	cl::vector<cl::Memory> m_clVector;

	uint m_posVBO;
	uint m_colorVBO;

	const char* m_fileName;
};

extern clParticles* g_psystem;
