#include <stdio.h>
#include <stdlib.h>
#include "SnowAvalancheSim.h"
#include "Camera.h"
#include "Scene.h" 
#include "OpenGL.h"

Camera * g_camera = 0;

const float HalfDegToRad = DegToRad/2.0f;

Camera::Camera() :
    m_bgColor(0,0,0),
    m_eye(0,0,0),
    m_viewDir(0,0,-1),
    m_up(0,1,0),
    m_lookAt(FLT_MAX, FLT_MAX, FLT_MAX),
    m_fov((45.)*(PI/180.)),
	m_dwFrames(0),
	m_dwCurrentTime(0),
	m_dwLastUpdateTime(0),
	m_dwElapsedTime(0)
{
    calcLookAt();
}


Camera::~Camera()
{

}


void
Camera::click(Scene* pScene) // Take a snapshot of the scene *CLICK*
{
	// Calculate the number of frames per one second:
	m_dwFrames++;
	m_dwCurrentTime = GetTickCount(); // Even better to use timeGetTime()
	m_dwElapsedTime = m_dwCurrentTime - m_dwLastUpdateTime;

	if(m_dwElapsedTime >= 200)
	{
		printf("FPS: %u\r", (uint)(m_dwFrames * 1000.0 / m_dwElapsedTime));
		m_dwFrames = 0;
		m_dwLastUpdateTime = m_dwCurrentTime;
	}

    calcLookAt();

    glDrawBuffer(GL_BACK);
    pScene->openGL(this); // Render the scene in opengl.

}


void
Camera::calcLookAt()
{
    // this is true when a "lookat" is not used in the config file
    if (m_lookAt.x != FLT_MAX)
    {
        setLookAt(m_lookAt);
        m_lookAt.set(FLT_MAX, FLT_MAX, FLT_MAX);
    }
}


void
Camera::drawGL()
{
    // set up the screen with our camera parameters
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	gluPerspective(fov(), g_scene->windowWidth()/(float)g_scene->windowHeight(), 0.01, 10000);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    Vector3 vCenter = eye() + viewDir();
    gluLookAt(eye().x, eye().y, eye().z, vCenter.x, vCenter.y, vCenter.z, up().x, up().y, up().z);
}
