#include "SnowAvalancheSim.h"
#include "Scene.h"
#include "Camera.h"
#include "shaders.h"

Scene * g_scene = 0;

void
Scene::openGL(Camera *cam) // Render scene using openGL
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    cam->drawGL(); // Setup the screen with camera parameters.

	glEnable( GL_POINT_SPRITE_ARB );
	glTexEnvi( GL_POINT_SPRITE_ARB, GL_COORD_REPLACE_ARB, GL_TRUE );
	glEnable(GL_VERTEX_PROGRAM_POINT_SIZE_NV);
	glDepthMask(GL_TRUE);
	glEnable(GL_DEPTH_TEST);

	glUseProgram(m_program);
	glUniform1f( glGetUniformLocation(m_program, "pointScale"), m_windowHeight / tanf(cam->fov() * m_fHalfViewRadianFactor));
	glUniform1f( glGetUniformLocation(m_program, "pointRadius"), g_psystem->getParticleRadius() );

	glColor3f(1.0, 1.0, 1.0);
	
	glBindBufferARB(GL_ARRAY_BUFFER_ARB, g_psystem->getPosVBO());
    glVertexPointer(4, GL_FLOAT, 0, 0);
    glEnableClientState(GL_VERTEX_ARRAY);            

		glBindBufferARB(GL_ARRAY_BUFFER_ARB, g_psystem->getColorVBO());
        glColorPointer(4, GL_FLOAT, 0, 0);
        glEnableClientState(GL_COLOR_ARRAY);

	glDrawArrays(GL_POINTS, 0, g_psystem->getNumParticles());

    glBindBufferARB(GL_ARRAY_BUFFER_ARB, 0);
    glDisableClientState(GL_VERTEX_ARRAY); 
    glDisableClientState(GL_COLOR_ARRAY);

	glUseProgram(0);
	glDisable(GL_POINT_SPRITE_ARB);

	// Reference points 
	glBegin(GL_POINTS);
		glColor3f(1.0, 0.0, 0.0);
		glVertex3f(-1.0, -1.0, -2.0);
		glVertex3f(-1.0, -1.0, 2.0);
		glVertex3f(-1.0, 3.0, 2.0);
		glVertex3f(9.0, 3.0, 2.0);
		glVertex3f(9.0, -1.0, 2.0);
		glVertex3f(9.0, -1.0, -2.0);
	glEnd();

	// Add cube
	glColor3f(1.0, 1.0, 1.0);
	glPushMatrix();
	glTranslatef(4.0, 1.0, 0.0);
	glScalef(5.0, 2.0, 2.0);
    glutWireCube(2.0);
	glPopMatrix();

    glutSwapBuffers();
}

void
Scene::preCalc() // Perform precalculation, if any
{
	GLuint vShader = glCreateShader(GL_VERTEX_SHADER);
	GLuint fShader = glCreateShader(GL_FRAGMENT_SHADER);

	glShaderSource(vShader, 1, &vertexShader, 0);
	glShaderSource(fShader, 1, &sphereShader, 0);

	glCompileShader(vShader);
	glCompileShader(fShader);

	m_program = glCreateProgram();

	glAttachShader(m_program, vShader);
	glAttachShader(m_program, fShader);

	glLinkProgram(m_program);

#if !defined(__APPLE__) && !defined(MACOSX)
    glClampColorARB(GL_CLAMP_VERTEX_COLOR_ARB, GL_FALSE);
    glClampColorARB(GL_CLAMP_FRAGMENT_COLOR_ARB, GL_FALSE);
#endif

	m_fHalfViewRadianFactor = (0.5f * 3.1415926535f/180.0f);
}