#include <math.h>
#include "SnowAvalancheSim.h" // contains simulation parameters.
#include "Scene.h"
#include "Camera.h"

#include "SimWindow.h" // OpenGL handler

#include "clAdmin.h" // opencl management
#include "clParticles.h" // particle system management
#include "clSort.h" // sorting procedure management


void
setupScene() {
	g_camera = new Camera;
    g_scene = new Scene;
    
	g_scene->resizeWindow(1024, 728);

    // set up the camera
    g_camera->setBGColor(Vector3(1.0f, 1.0f, 1.0f));
    g_camera->setEye(Vector3(0, 0, -4));
    g_camera->setLookAt(Vector3(0, 0, 0));
    g_camera->setUp(Vector3(0, 1, 0));
    g_camera->setFOV(45);   
}

void
makeParticleSystem() {
	g_clAdmin = new clAdmin();

	g_psystem = new clParticles(numParticles, GRID_SIZE, particleRadius);
	g_psystem->init();
}

int
main(int argc, char*argv[])
{
	srand ( 2011 );

	setupScene(); // set up opengl and display

    SimWindow simulation(&argc, argv);

    g_scene->preCalc();

	makeParticleSystem(); // create particle system

    simulation.mainLoop(); // run simulation

    return 0; // never executed
}